﻿using LiangLiSystem.Services.Helpers;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Traffic.Data.ViewModels;
using Traffic.Service.Interfaces;

namespace Traffic.Api.Controllers
{
    /// <summary>
    /// 違規事件
    /// </summary>
    [Route("api/eventType")]
    [Authorize]
    [ApiController]
    public class EventTypeController : ControllerBase
    {
        private readonly IEventTypeService _service;

        public EventTypeController(IEventTypeService service)
        {
            _service = service;
        }

        /// <summary>
        /// 取得所有違規列表 (EventType)
        /// </summary>
        /// <returns></returns>
        [HttpGet("all")]
        public IActionResult GetEventTypes()
        {
            try
            {
                var result = new Response();
                result.Result = _service.GetEventTypes();
                return Ok(result);
            }
            catch (Exception)
            {
                return BadRequest();
            }
        }

        /// <summary>
        /// 取得所有違規列表 (EventType1)
        /// </summary>
        /// <returns></returns>
        [HttpGet("1/all")]
        public IActionResult GetEventType1s()
        {
            try
            {
                var result = new Response();
                result.Result = _service.GetEventType1s();
                return Ok(result);
            }
            catch (Exception)
            {
                return BadRequest();
            }
        }

        /// <summary>
        /// 取得 EventType 列表
        /// </summary>
        /// <param name="searchModel"></param>
        /// <returns></returns>
        [HttpGet("")]
        public IActionResult GetEventTypes([FromQuery] SearchModelViewModel searchModel)
        {
            try
            {
                var result = new Response();
                var model = Util.GetSearchModel(searchModel);
                result.Result = _service.GetEventTypes(model);
                return Ok(result);
            }
            catch (Exception)
            {
                return BadRequest();
            }
        }

        /// <summary>
        /// 取得 EventType by id
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet("{id}")]
        public IActionResult GetEventType(int id)
        {
            try
            {
                var result = new Response();
                result.Result = _service.GetEventTypeById(id);
                return Ok(result);
            }
            catch (Exception)
            {
                return BadRequest();
            }
        }

        /// <summary>
        /// 新增 EventType 
        /// </summary>
        /// <param name="vm"></param>
        /// <returns></returns>
        [HttpPost("")]
        public IActionResult InsertEventType(EventTypeViewModel vm)
        {
            try
            {
                var result = _service.InsertEventType(vm);
                return result.Success ? Ok(result) : BadRequest(result);
            }
            catch (Exception)
            {
                return BadRequest();
            }
        }

        /// <summary>
        /// 修改 EventType 
        /// </summary>
        /// <param name="id">EventType 的流水號</param>
        /// <param name="vm"></param>
        /// <returns></returns>
        [HttpPut("{id}")]
        public IActionResult UpdateEventType(int id, EventTypeViewModel vm)
        {
            try
            {
                var result = _service.UpdatEventType(id, vm);
                return result.Success ? Ok(result) : BadRequest(result);
            }
            catch (Exception)
            {
                return BadRequest();
            }
        }

        /// <summary>
        /// 刪除 EventType 
        /// </summary>
        /// <param name="id">EventType 的流水號</param>
        /// <returns></returns>
        [HttpDelete("{id}")]
        public IActionResult DeleteEventType(int id)
        {
            try
            {
                var result = _service.DeleteEventType(id);
                return result.Success ? Ok(result) : BadRequest(result);
            }
            catch (Exception)
            {
                return BadRequest();
            }
        }
    }
}
