﻿using Dapper;
using System.Collections.Generic;
using System.Data;
using Traffic.Data.Models;
using Traffic.Repository.Interfaces;

namespace Traffic.Repository
{
    public class EventTypeRepository : IEventTypeRepository
    {
        public IDbTransaction Transaction { get; }
        public IDbConnection Connection => Transaction.Connection;

        public EventTypeRepository(IDbTransaction transaction)
        {
            Transaction = transaction;
        }

        public IEnumerable<EventTypeData> GetEventTypes()
        {
            var sql = $"SELECT * FROM EventType";
            return Connection.Query<EventTypeData>(sql, null, Transaction);
        }

        public IEnumerable<EventTypeData> GetEventType1s()
        {
            var sql = $"SELECT * FROM EventType1";
            return Connection.Query<EventTypeData>(sql, null, Transaction);
        }

        public EventTypeData GetEventTypeById(int id)
        {
            var sql = $"SELECT * FROM EventType Where {nameof(EventTypeData.Id)} = @{nameof(EventTypeData.Id)}";
            return Connection.QueryFirstOrDefault<EventTypeData>(sql, new { id }, Transaction);
        }

        public EventTypeData GetEventTypeByEventType(string eventType)
        {
            var sql = $"SELECT * FROM EventType Where {nameof(EventTypeData.EventType)} = @{nameof(EventTypeData.EventType)}";
            return Connection.QueryFirstOrDefault<EventTypeData>(sql, new { eventType }, Transaction);
        }

        public bool InsertEventType(EventTypeData company)
        {
            var sql = $@"INSERT INTO EventType (
                                     {nameof(EventTypeData.EventType)}, 
                                     {nameof(EventTypeData.EventName)})
	                          VALUES(@{nameof(EventTypeData.EventType)}, 
                                     @{nameof(EventTypeData.EventName)})";

            var result = Connection.Execute(sql, company, Transaction);
            return result == 1;
        }

        public bool UpdateEventType(EventTypeData company)
        {
            var sql = $@"UPDATE Eventtype
	                        SET {nameof(EventTypeData.EventType)} = @{nameof(EventTypeData.EventType)}, 
                                {nameof(EventTypeData.EventName)} = @{nameof(EventTypeData.EventName)}  
	                      WHERE {nameof(EventTypeData.Id)} = @{nameof(EventTypeData.Id)}";
            var result = Connection.Execute(sql, company, Transaction);
            return result == 1;
        }

        public bool DeleteEventType(int id)
        {
            var sql = $@"DELETE FROM Eventtype WHERE  {nameof(EventTypeData.Id)} = @{nameof(EventTypeData.Id)}";
            var result = Connection.Execute(sql, new { id }, Transaction);
            return result == 1;
        }
    }
}
