﻿using LiangLiSystem.Services.Helpers;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using Traffic.Data.Models;
using Traffic.Data.ViewModels;
using Traffic.Repository.Infrastructures;
using Traffic.Service.Interfaces;

namespace Traffic.Service.Implements
{
    public class LoginService : ILoginService
    {
        private readonly IUnitOfWork _unitOfWork;

        public LoginService(IUnitOfWork unitOfWork)
        {
            _unitOfWork = unitOfWork;
        }

        public AccountUserViewModel Login(string account, string password, string ip = "")
        {
            SHA256 sha256 = new SHA256CryptoServiceProvider();//建立一個SHA256
            byte[] source = Encoding.Default.GetBytes(password);//將字串轉為Byte[]
            byte[] crypto = sha256.ComputeHash(source);//進行SHA256加密
            var pwdHash = Convert.ToBase64String(crypto);//把加密後的字串從Byte[]轉為字串

            var data = _unitOfWork.AccountRepository.GetAccountUserByAccount(account);
            if (data == null)
            {
                return null;
            }

            var log = new UserLoginLog
            {
                UserId = data.Id,
                LoginIP = ip,
                LoginOn = Util.GetDateTimeNow(),
                Status = data.Password != pwdHash ? 0 : 1
            };

            _unitOfWork.AccountRepository.InsertUserLoginLog(log);

            if (data.Password != pwdHash)
            {
                var count = data.ErrorCount + 1;
                _unitOfWork.AccountRepository.UpdateUserErrorCount(data.Id, count);
                _unitOfWork.Complete();
                var viewModel = new AccountUserViewModel
                {
                    Id = data.Id,
                    Account = data.Account,
                    Name = data.UserName,
                    Email = data.Email,
                    Role = data.RoleCodeId,
                    ErrorCount = data.ErrorCount + 1,
                    ChangePwdOn = data.ChangePwdOn,
                    Status = data.Status
                }; 
                return viewModel;
            }
            else
            {
                _unitOfWork.AccountRepository.UpdateUserErrorCount(data.Id, 0);
                _unitOfWork.Complete();
                var viewModel = new AccountUserViewModel
                {
                    Id = data.Id,
                    Account = data.Account,
                    Name = data.UserName,
                    Email = data.Email,
                    Role = data.RoleCodeId,
                    ErrorCount = 0,
                    ChangePwdOn = data.ChangePwdOn,
                    Status = data.Status
                };

                return viewModel;
            }
        }
    }
}
